package com.cloud.kicc.common.data.entity;

import cn.hutool.core.util.ObjectUtil;
import com.cloud.kicc.common.core.constant.SecurityConstants;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.authority.SimpleGrantedAuthority;

import java.time.LocalDateTime;
import java.util.List;

/**
 *<p>
 * 扩展用户数据
 *</p>
 *
 * @Author: wangxiang4
 * @Since: 2023/8/16
 */
@Getter
@Setter
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = false)
@ApiModel(description = "完整用户信息")
public class KiccUser extends CasUser {

	@ApiModelProperty("扩展用户ID")
	private String id;

	@ApiModelProperty("CAS用户ID")
	private String casUserId;

    @ApiModelProperty("用户类型 {@link com.cloud.kicc.system.api.enums.UserTypeEnum }")
    private String userType;

    @ApiModelProperty("机构ID")
    private String deptId;

    @ApiModelProperty("机构名称")
    private String deptName;

    @ApiModelProperty("地图标记点位置图片旋转值")
    private Double mapOrientation;

    @ApiModelProperty("地图设计器默认中心点位置")
    private String mapCenter;

    @ApiModelProperty("帐号状态（0正常 1停用）")
    private String status;

    @ApiModelProperty("指定登录后首页跳转")
    private String homePath;

    @ApiModelProperty("角色ID集合")
    private String[] roleIds;

    @ApiModelProperty("菜单按钮权限")
    private String[] permissions;

    @ApiModelProperty("多租户ID集合")
    private String[] tenantIds;

    @ApiModelProperty("创建ID")
    private String createById;

    @ApiModelProperty("创建人")
    private String createByName;

    @ApiModelProperty("创建时间")
    private LocalDateTime createTime;

    @ApiModelProperty("更新ID")
    private String updateById;

    @ApiModelProperty("更新人")
    private String updateByName;

    @ApiModelProperty("更新时间")
    private LocalDateTime updateTime;

	public KiccUser() {
		super(SecurityConstants.MOCK_USERNAME, SecurityConstants.MOCK_PASSWORD, AuthorityUtils.createAuthorityList());
	}

	@JsonCreator
	public KiccUser(@JsonProperty("username") String username,
                    @JsonProperty("password") String password,
                    @JsonProperty("enabled") boolean enabled,
                    @JsonProperty("accountNonExpired") boolean accountNonExpired,
                    @JsonProperty("credentialsNonExpired") boolean credentialsNonExpired,
                    @JsonProperty("accountNonLocked") boolean accountNonLocked,
                    @JsonProperty("authorities") List<SimpleGrantedAuthority> authorities) {
		super(ObjectUtil.defaultIfBlank(username, SecurityConstants.MOCK_USERNAME),
              ObjectUtil.defaultIfBlank(password, SecurityConstants.MOCK_PASSWORD),
              enabled,
              accountNonExpired,
              credentialsNonExpired,
              accountNonLocked,
              ObjectUtil.defaultIfNull(authorities, AuthorityUtils.createAuthorityList()));
	}

}
