package com.cloud.kicc.common.datasource.dynamic;

import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.provider.AbstractJdbcDataSourceProvider;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DataSourceProperty;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.cloud.kicc.common.datasource.support.DynamicDataSourceConstant;
import com.cloud.kicc.common.datasource.util.ConnUtil;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

/**
 *<p>
 * 动态数据源初始加载
 *</p>
 *
 * @Author: wangxiang4
 * @Since: 2023/7/3
 */
public class DynamicDataSourceJdbcProvider extends AbstractJdbcDataSourceProvider {

	private final String driverClassName;
	private final String url;
	private final String username;
	private final String password;
	private final DynamicDataSourceProperties dynamicDataSourceProperties;

	public DynamicDataSourceJdbcProvider(DynamicDataSourceProperties dynamicDataSourceProperties,
										 String driverClassName,
										 String url,
										 String username,
										 String password) {
		super(driverClassName, url, username, password);
		this.dynamicDataSourceProperties = dynamicDataSourceProperties;
		this.driverClassName = driverClassName;
		this.url = url;
		this.username = username;
		this.password = password;
	}

	@Override
	protected Map<String, DataSourceProperty> executeStmt(Statement statement) throws SQLException {
		// 构建数据源集合
		Map<String, DataSourceProperty> map = new HashMap<>(16);
		// 构建主数据源
		DataSourceProperty masterProperty = new DataSourceProperty();
		masterProperty.setDriverClassName(driverClassName);
		masterProperty.setUrl(url);
		masterProperty.setUsername(username);
		masterProperty.setPassword(password);
		map.put(dynamicDataSourceProperties.getPrimary(), masterProperty);
		// 构建yml数据源
		Map<String, DataSourceProperty> datasource = dynamicDataSourceProperties.getDatasource();
		if (!datasource.isEmpty()) {
			datasource.remove(dynamicDataSourceProperties.getPrimary());
			map.putAll(datasource);
		}
		// 构建动态数据源
		ResultSet rs = statement.executeQuery(DynamicDataSourceConstant.DYNAMIC_DATASOURCE_GROUP_STATEMENT);
		while (rs.next()) {
			String id = rs.getString("id");
			String name = rs.getString("name");
			String driver = rs.getString("driverClass");
			String url = rs.getString("url");
			String username = rs.getString("username");
			String password = rs.getString("password");
			try {
				if (StrUtil.isAllNotBlank(id, driver, url, username, password)) {
					// 测试链接是否生效
					Boolean result = ConnUtil.dbTest(driver, url, username, password);
					if (result) {
						DataSourceProperty jdbcProperty = new DataSourceProperty();
						// 设置SQL链接
						jdbcProperty.setDriverClassName(driver);
						jdbcProperty.setUrl(url);
						jdbcProperty.setUsername(username);
						jdbcProperty.setPassword(password);
						map.put(name, jdbcProperty);
					}
				}
			} catch (Exception e) {
				System.err.printf("数据源:"+ name + "初始化失败!");
			}
		}
		return map;
	}
}
