package com.cloud.kicc.common.security.config;

import com.cloud.kicc.common.security.exp.KiccLocalResourceServerTokenServices;
import com.cloud.kicc.common.security.exp.PermissionService;
import com.cloud.kicc.common.security.exp.PermitAllUrlProperties;
import com.cloud.kicc.common.security.exp.ResourceAuthExceptionEntryPoint;
import com.cloud.kicc.common.security.override.KiccBearerTokenExtractor;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.security.oauth2.provider.token.ResourceServerTokenServices;

/**
 *<p>
 * 扩展资源服务器自动配置
 *</p>
 *
 * @Author: wangxiang4
 * @Date: 2022/2/17
 */
@EnableConfigurationProperties(PermitAllUrlProperties.class)
public class ResourceServerAutoConfiguration {

	/** 校验接口是否存在权限 */
	@Bean("pms")
	public PermissionService permissionService() {
		return new PermissionService();
	}

	/** 对公开权限的请求获取token不进行校验 */
	@Bean
	public KiccBearerTokenExtractor kiccBearerTokenExtractor(PermitAllUrlProperties urlProperties) {
		return new KiccBearerTokenExtractor(urlProperties);
	}

	/** 细粒化客户端认证异常处理 */
	@Bean
	@ConditionalOnMissingBean
	public ResourceAuthExceptionEntryPoint resourceAuthExceptionEntryPoint() {
		return new ResourceAuthExceptionEntryPoint();
	}

	/** 扩展资源服务器令牌服务 */
	@Bean
	@Primary
	public ResourceServerTokenServices resourceServerTokenServices() {
		return new KiccLocalResourceServerTokenServices();
	}

}
