package com.cloud.kicc.common.security.service;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.cloud.kicc.common.core.constant.CommonConstants;
import com.cloud.kicc.common.core.constant.SecurityConstants;
import com.cloud.kicc.common.data.entity.CasUser;
import com.cloud.kicc.common.data.entity.SsoUser;
import com.cloud.kicc.common.security.exception.SecurityCheckedException;
import lombok.SneakyThrows;
import org.springframework.core.Ordered;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;

/**
 *<p>
 * 用户详细信息服务
 *</p>
 *
 * @Author: wangxiang4
 * @Date: 2022/2/17
 */
public interface KiccUserDetailsService extends UserDetailsService, Ordered {

	/**
	 * 是否支持此客户端校验
	 * @param clientId 目标客户端
	 * @return true/false
	 */
	default boolean support(String clientId, String grantType) {
		return true;
	}

	/**
	 * 排序值 默认取最大的
	 * @return 排序值
	 */
	@Override
	default int getOrder() {
		return 0;
	}

	/**
	 * 构建userDetails
	 * @param user 用户信息
	 * @return UserDetails
	 */
	@SneakyThrows
	default UserDetails getUserDetails(SsoUser user) {
		if (ObjectUtil.isEmpty(user)) {
			throw new SecurityCheckedException("SSO User not found, Try again after registration.");
		}

        // 构造security用户
		return new CasUser(
				user.getUserName(),
				SecurityConstants.BCRYPT + user.getPassword(),
				true,
				true,
				true,
				StrUtil.equals(user.getStatus(), CommonConstants.STATUS_NORMAL),
				AuthorityUtils.createAuthorityList(),
				user.getId(),
				user.getNickName(),
				user.getEmail(),
				user.getPhone(),
				user.getSex(),
				user.getAvatar(),
				user.getLoginIp(),
				user.getLoginTime(),
				user.getStatus(),
				user.getCreateById(),
				user.getCreateByName(),
				user.getCreateTime(),
				user.getUpdateById(),
				user.getUpdateByName(),
				user.getUpdateTime(),
				user.getRemarks(),
				null,
				user.getTenantId()
		);
	}

	/**
	 * 通过用户实体查询
	 * @param casUser user
	 */
	default UserDetails loadUserByUser(CasUser casUser) {
		return this.loadUserByUsername(casUser.getUsername());
	}

}
