package com.cloud.kicc.common.security.service.impl;

import com.cloud.kicc.common.core.constant.SecurityConstants;
import com.cloud.kicc.common.data.entity.CasUser;
import com.cloud.kicc.common.data.entity.SsoUser;
import com.cloud.kicc.common.security.service.KiccUserDetailsService;
import com.cloud.kicc.common.security.template.UserProviderTemplate;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;

/**
 *<p>
 * 用户详细信息
 *</p>
 *
 * @Author: wangxiang4
 * @Date: 2022/2/17
 */
@Slf4j
@RequiredArgsConstructor
public class KiccAppUserDetailsServiceImpl implements KiccUserDetailsService {

	@Autowired(required = false)
	private UserProviderTemplate userProviderTemplate;

	/**
	 * 手机号登录
	 * @param phone 手机号
	 */
	@Override
	@SneakyThrows
	public UserDetails loadUserByUsername(String phone) {
		SsoUser result = userProviderTemplate.selectByPhone(phone);
        return getUserDetails(result);
	}

	/**
	 * 通过用户实体查询
	 * @param casUser user
	 */
	@Override
	public UserDetails loadUserByUser(CasUser casUser) {
		return this.loadUserByUsername(casUser.getPhone());
	}

	/**
	 * 是否支持此客户端校验
	 * @param clientId 目标客户端
	 * @return true/false
	 */
	@Override
	public boolean support(String clientId, String grantType) {
		return SecurityConstants.APP.equals(grantType);
	}

}
