package com.cloud.kicc.common.security.xss;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.servlet.DispatcherType;
import java.util.HashMap;
import java.util.Map;

/**
 *<p>
 * XSS-Filter注册
 *</p>
 *
 * @Author: wangxiang4
 * @Date: 2022/2/19
 */
@Configuration(proxyBeanMethods = false)
@EnableConfigurationProperties(XssProperties.class)
public class XssFilterAutoConfiguration {

    @Bean
    public FilterRegistrationBean xssFilterRegistration(XssProperties xssProperties) {
        FilterRegistrationBean registration = new FilterRegistrationBean();
        registration.setDispatcherTypes(DispatcherType.REQUEST);
        registration.setFilter(new XssFilter());
        registration.addUrlPatterns(StrUtil.splitToArray(xssProperties.getUrlPatterns(), ","));
        registration.setName("xssFilter");
        registration.setOrder(FilterRegistrationBean.HIGHEST_PRECEDENCE);
        Map<String, String> initParameters = new HashMap();
        initParameters.put("excludes", ObjectUtil.defaultIfNull(xssProperties.getExcludes(), ""));
        initParameters.put("enabled", ObjectUtil.defaultIfNull(xssProperties.getEnabled(), ""));
        registration.setInitParameters(initParameters);
        return registration;
    }

}
